/******************************************************************************\
* $Id:$
*
* Copyright(c) 2015 SAS Institute Inc., Cary, NC, USA. All Rights Reserved.
*
* Name: fda_openCdisc_reconciliation.sas
*
* Purpose: identify the OpenCDISC check that corresponds with an FDA Rule
*
* Author: Ben Bocchicchio, Frank Roediger
*
* Support: SAS(r) Solutions OnDemand
*
* Input: 
*
* Output: N/A
*
* Parameters: N/A
*
* Dependencies/Assumptions: 
*   - OpenCDISC Checks have been read from the OpenCDISC Web site and converted 
*     to a SAS data set called openCDISCTests
*   - FDA Rules have been read from their spreadsheet and converted to a 
*     SAS data set called fdaRules_excel
*
* Usage: 
*
* History:
* ddmmmyyyy userid         description                             (Change Code)
* 19FEB2015 FrankRoediger  initial development
\******************************************************************************/
ods html close;
ods listing;


%*******************************************************************************
%* prepare the OpenCDISC data set to be match/merged with FDA data set
%******************************************************************************;
data openCDISCTests_313;
   set openCDISCTests;

   length openCdiscID  $ 6
          matchString  $ 535;

   openCdiscID=strip(checkID);
   matchString=upcase(compbl(checkDescription));

   keep matchString
        openCdiscID
        checkmessage
        checkDescription
       ;

run;
proc sort data=openCDISCTests_313;
   by matchString;
run;


%*******************************************************************************
%* prepare the FDA data set to be match/merged with OpenCDISC data set 
%******************************************************************************;
data fdaRules_313;
   set fdaRules_excel;

   if _3_1_3 eq 'X';

   length fdaID        $ 7
          matchString  $ 535;

   fdaID=strip(FDA_Rule_ID);
   matchString=upcase(compbl(description));

   keep matchString
        fdaID
        description
        message
        severity
       ;

run;
proc sort data=fdaRules_313;
   by matchString;
run;


%*******************************************************************************
%* initial m/m identifies exact matches 
%******************************************************************************;
data match0
     both
     onlyL
     onlyR;
   merge fdaRules_313 (in=inL)
         openCDISCTests_313 (in=inR);
   by matchString;

   if inL then 
      if inR then do;
         output both;
         output match0;
      end;
      else
         output onlyL;
   else
      output onlyR;

run;


%*******************************************************************************
%* create a list of FDA rules that have not been matched yet 
%******************************************************************************;
options ls=200 ps= 78 pageno=1;
data _null_;
   length rptLine $ 175
          parseLine $ 900;

   set onlyL;

   file print notitles;

   put / @ 12 fdaID @;

   parseLine=description;

   rptLine=substr(parseLine,1,175);
   nextStart=0;
   do i=1 to 20 while (rptLine gt ' ');
      if substr(rptLine,175,1) ne ' ' then do;
         do j=175 to 1 by -1 until(substr(rptLine,j,1) eq ' ');
         end;
         rptLine=substr(rptLine,1,j);
      end;
      else
         j=175;
      put @ 25 rptLine;
      nextStart=nextStart+j;
      rptLine=substr(parseLine,nextStart,175);
   end;
     
run;


%*******************************************************************************
%* create a list of OpenCDISC rules that have not been matched yet 
%******************************************************************************;
options ls=200 ps= 78 pageno=1;
data _null_;
   length rptLine $ 175
          parseLine $ 750;

   set onlyR;

   file print notitles;

   put / @ 12 openCdiscID @;

   parseLine=checkDescription;

   rptLine=substr(parseLine,1,175);
   nextStart=0;
   do i=1 to 20 while (rptLine gt ' ');
      if substr(rptLine,175,1) ne ' ' then do;
         do j=175 to 1 by -1 until(substr(rptLine,j,1) eq ' ');
         end;
         rptLine=substr(rptLine,1,j);
      end;
      else
         j=175;
      put @ 25 rptLine;
      nextStart=nextStart+j;
      rptLine=substr(parseLine,nextStart,175);
   end;
     
run;


%*******************************************************************************
%* from the unmatched lists, manually associate FDA Rules with OpenCDISC Checks,
%*    based on strong similarities between the descriptions 
%******************************************************************************;
data match01;
   input @  1 fdaID $char7.
         @ 10 cdisc $char6.;
   
   datalines;
FDAC027  SD0058
FDAC033  SD0063
FDAC040  SD0064
FDAC044  SD0005
FDAC052  SD0069
FDAC064  SD0071
FDAC082  SD0015
FDAC084  SD0007
FDAC104  SD0065
FDAC105  SD1023
FDAC113  SD0006
FDAC122  SD0031
FDAC183  SD0035
FDAC199  SD2002
FDAC200  SD2003
FDAC206  SD0009
FDAC216  SD1021
FDAC346  SD0008
FDAC350  SD2007
FDAC351  SD2008
FDAC353  SD2009
FDAC354  SD2010
FDAC356  SD2011
FDAC357  SD2012
FDAC359  SD2013
FDAC361  SD2015
FDAC363  SD2016
;
run;

%*******************************************************************************
%* create a $toCDISC format to associate FDA Rules with OpenCDISC Checks, based
%*    on strong similarities between the descriptions 
%******************************************************************************;
data cntlIn1;
   set match01 
         (rename=(fdaID=start
                  cdisc=label
                 )
         )
      end=eof;
   retain fmtname '$toCDISC';
   
   output;

   if eof then do;
      hlo='O';
      label='X';
      output;
   end;

run;
proc format cntlIn=cntlIn1;
run;


%*******************************************************************************
%* use the $toCDISC format to associate FDA Rules with OpenCDISC Checks
%******************************************************************************;
data fdaRules2;
   set onlyL;

   length openCdiscID $ 6;

   openCdiscID=put(fdaID,$toCdisc.);

   keep fdaID
        openCdiscID
        description
        severity
       ;
run;
proc sort data=fdaRules2;
   by openCdiscID;
run; 
proc sort data=onlyR
            (keep=checkDescription
                  openCdiscID
            )
          out =Opencdisctests_313_2;
   by openCdiscID;
run;


%*******************************************************************************
%* m/m unmatched FDA Rules with unmatched OpenCDISC Checks, based on the 
%*    OpenCDISC ID (ascribed to unmatched FDA Rules by the $toCdisc. format)
%******************************************************************************;
data match1
     both
     onlyL
     onlyR;
   merge fdaRules2            (in=inL)
         Opencdisctests_313_2 (in=inR);
   by openCdiscID;

   if inL then 
      if inR then do;
         output both;
         output match1;
      end;
      else
         output onlyL;   /* unmatched FDA Rules */
   else
      output onlyR;      /* unmatched OpenCDISC Rules */

run;


%*******************************************************************************
%* combine the MATCH0 and MATCH1 data sets  
%******************************************************************************;
data matched;
   length reason $ 20;
   set match0 (in=in1)
       match1 (in=in2);

   if in1 then reason='Exact Match';
   else        reason='Strong Similarity';

run;


